# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2010
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Eole - 2015 (Ministère Écologie - CPII)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr
###########################################################################
"""
Gestion des quotas disque des utilisateurs
"""
from pyeole.process import system_out, system_code
import re
import os

ORDERS = ('K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
VALUE_RE = re.compile(r'^(?P<value>[-+]?[0-9]+[.,]?[0-9]?)\s*(?P<unit>[KkMmGgTtPpEeZzYy]?[Bbo]?)$')
HARD_MULTIPLIER = 1
DEFAULT_UNIT = 'M'


def human_to_kilobytes(size):
    """
    renvoie la valeur value convertie sans unité
    :param value: valeur avec unité
    :type value: objet re.Match
    """
    value = float(size.group('value'))
    unit = size.group('unit')
    if len(unit) == 2:
        unit = unit[0].upper()
    if unit not in ORDERS:
        unit = DEFAULT_UNIT
    value = int(value * 2**(10*ORDERS.index(unit)))
    return value


def kilobytes_to_blocks(value, path):
    """
    renvoie le nombre de block le plus proche de la taille fournie
    :param value: taille
    :type value: int
    :param path: chemin
    :type path: str
    """
    block_size = os.statvfs(path)[1]
    block_nb = ((value / block_size) +
                (1 if value % block_size > block_size/2 else 0))
    return block_nb


def kilobytes_to_human(size):
    """
    renvoie la valeur value sous la forme humainement lisible la plus compacte
    (avec la plus grande unité possible)
    :param value: valeur sans unité
    :type value: int
    """
    value_repr = [(o, round(float(size)/2**(10*i), 1))
                  for i, o in enumerate(ORDERS)]
    values = [value for value in value_repr
              if size == human_to_kilobytes(VALUE_RE.search('{1} {0}'.format(*value))) and
              int(value[1]) > 0]
    value = values[-1] if len(values) > 0 else ('M', 0)
    return '{0} {1}{2}'.format(value[1], value[0], 'o')


def blocks_to_kilobytes(blocks, path):
    """
    renvoie la taille définie par le nombre de blocs pour un chemin donné.
    :param blocks: nombre de blocs
    :type blocks: int
    :param path: chemin
    :type path: str
    """
    block_size = os.statvfs(path)[1]
    return blocks * block_size


def is_quota(quota):
    """
    renvoie vrai si le paramètre quota est interprétable comme un quota
    :param quota: représentation du quota
    :type quota: str
    """
    quota_re = VALUE_RE.match(quota.strip())
    if quota_re:
        quota_bool = True
    else:
        quota_bool = False
    return quota_bool


def get_quota(login):
    """
    renvoie le quota disque pour un utilisateur
    """
    cmd = ['/usr/bin/quota', '-w', login]
    res = system_out(cmd, container='fichier')[1].splitlines()
    if len(res) < 3:
        user_quota = 0
    else:
        nb_blocks = res[2].split()[2]
        # on retourne la limite en Mo
        user_quota = int(nb_blocks)/1024
    return user_quota


def get_ug_quota(name, path, mode):
    """
    :param name: utilisateur pour lequel le quota est défini
    :type name: str
    :param path: chemin auquel s'applique le quota
    :type path: str
    :param mode: type de quota
    :type mode: 'group'|'user'
    """
    cmd = ['/usr/sbin/quotatool', '-d',
           '-u' if mode == 'user' else '-g', name,
           path]
    res = system_out(cmd, container='fichier')[1].split()
    user_quota = kilobytes_to_human(int(res[3]))
    return user_quota


def get_user_quota(user, path='/home'):
    """
    renvoie le quota pour un utilisateur
    :param user: utilisateur
    :type user: str
    :rtype: int
    """
    return get_ug_quota(user, path, 'user')


def get_group_quota(group, path='/home'):
    """
    renvoie le quota pour un group
    :param group: groupe
    :type group: str
    :rtype: int
    """
    return get_ug_quota(group, path, 'group')


def set_ug_quota(name, quota, path, mode):
    """
    renvoie le code de retour de la commande système pour assigner les quotas.
    :param name: utilisateur pour lequel le quota est défini
    :type name: str
    :param quota: valeur du quota
    :type quota: str
    :param path: chemin auquel s'applique le quota
    :type path: str
    :param mode: type de quota
    :type mode: 'group'|'user'
    :rtype: int
    """
    quota_re = VALUE_RE.match(quota.strip())
    if quota_re:
        kilobytes = human_to_kilobytes(quota_re)
    else:
        raise ValueError("Le quota disque fourni n'est pas exploitable")
    if not os.path.isdir(path):
        raise ValueError("Le chemin fourni n'est pas un dossier")
    if mode not in ['group', 'user']:
        raise ValueError("Le mode doit être 'group' ou 'user'")

    soft_limit = kilobytes
    unit = "K"

    # quotatool n'accepte pas les arguments trop longs ?
    while soft_limit > 999999999:
        soft_limit = soft_limit / 2**10
        unit = ORDERS[ORDERS.index(unit)+1]
    unit = '{0}b'.format(unit)

    hard_limit = soft_limit*HARD_MULTIPLIER
    cmd = ['/usr/sbin/quotatool', '-b',
           '-q', "{0}{1}".format(soft_limit, unit),
           '-l', "{0}{1}".format(hard_limit, unit),
           '-u' if mode == 'user' else '-g', name,
           path]
    return system_code(cmd, container='fichier')


def set_group_quota(group, quota="0", path='/home'):
    """
    renvoie le code de retour de la commande système pour assigner les quotas.
    :param group: groupe pour lequel le quota est défini
    :type group: str
    :param quota: valeur du quota
    :type quota: str
    :param path: chemin auquel s'applique le quota
    :type path: str
    :rtype: int
    """
    return set_ug_quota(group, quota, path, 'group')


def set_user_quota(user, quota="0", path='/home'):
    """
    renvoie le code de retour de la commande système pour assigner les quotas.
    :param user: utilisateur pour lequel le quota est défini
    :type user: str
    :param quota: valeur du quota
    :type quota: str
    :param path: chemin auquel s'applique le quota
    :type path: str
    :rtype: int
    """
    return set_ug_quota(user, quota, path, 'user')


def set_quota(login, quota=0):
    """
    applique un nouveau quota disque
    pour un utilisateur
        @quota: int (str admis)
    """
    if type(quota) != int:
        try:
            quota = int(quota)
        except:
            raise Exception("valeur entière attendue pour un quota disque")
    cmd = ['/usr/sbin/quotatool', '-b',
           '-q', '%dM' % quota,
           '-l', '%dM' % (quota*2,),
           '-u', login, '/home']
    return system_code(cmd, container='fichier')
